/*
	Examples altered from http://obeybrew.com
*/

#include "huc.h"

/* defines make life easier and code more readable... */
#define DIR_LEFT 1
#define DIR_RIGHT 2

/* our new Bonk sprites! */
#incspr(bonk,"bonksprites.pcx",0,0,2,18);
#incpal(bonkpal,"bonksprites.pcx");

#incbin(bonk_map, "bonk_BAT.bin");
#incbin(bonk_bg, "bonk_DATA.bin");
#incbin(bonk_pal, "bonk_PAL.bin");

int bonkx, bonky, held, pressed, jumpspeed, jumpdelta;
char tics, frame, state, direction;

void spr_make(int spriteno, int spritex, int spritey, int spritepattern, int ctrl1, int ctrl2, int sprpal, int sprpri)
{
	spr_set(spriteno);
	spr_x(spritex);
	spr_y(spritey);
	spr_pattern(spritepattern);
	spr_ctrl(ctrl1,ctrl2);
	spr_pal(sprpal);
	spr_pri(sprpri);
}

void pause()
{
	while(1)
	{
		vsync();
		if (joytrg(0) & JOY_STRT) 
			return;
	}
}

void main()
{
	int colswap = 0;

	set_screen_size(SCR_SIZE_32x32);

 	tics = 0;
  	frame = 0;
  	bonkx = 104;
  	bonky = 176;
	state = 0;

	load_palette(16, bonkpal, 1);
	set_color(256, 0);
	load_vram(0x5000, bonk, 0x900);

	init_satb();
	spr_make(0, 104, 176, 0x5100, FLIP_MAS|SIZE_MAS, NO_FLIP|SZ_32x32, 0, 1);
	satb_update();

	load_background(bonk_bg, bonk_pal, bonk_map, 40, 30);

	while(1)
	{
		held = joy(0);
		pressed = joytrg(0);

		if (pressed & JOY_STRT)
			pause();
		else
			 bonk_state_machine();

		satb_update();

		if(colswap == 60)
		{
			SwapPalette(6, 4);
			SwapPalette(7, 8);
			
			colswap = 0;
		}
		colswap++;

		scroll(0, bonkx, 0, 0, 76, 0xC0);
		scroll(1, 2*bonkx, 76, 76, 160, 0xC0);
		scroll(2, 3*bonkx, 160, 160, 208, 0xC0);
		scroll(3, 4*bonkx, 208, 208, 240, 0xC0);

		vsync();
	}
}

void SwapPalette(int pal, int index)
{
	int pos;
	int tmpcol1 = 0;
	int tmpcol2 = 0;
	int tmpcol3 = 0;
	int tmpcol4 = 0;
	
	pos = index+pal*16;
	tmpcol1 = get_color(pos);
	tmpcol2 = get_color(pos+1);
	tmpcol3 = get_color(pos+2);
	tmpcol4 = get_color(pos+3);
		
	set_color(pos, tmpcol4);
	set_color(pos+1, tmpcol1);
	set_color(pos+2, tmpcol2);
	set_color(pos+3, tmpcol3);
}

void bonk_state_machine()
{
/*
  Bonk's State Machine:
*/
  if (state == 0) bonk_state_0(); //  0 - Idle (all buttons available)
  if (state == 1) bonk_state_1(); //  1 - Walking (all buttons available)
  if (state == 2) bonk_state_2(); //  2 - About to jump (all buttons disabled)
  if (state == 3) bonk_state_3(); //  3 - Jumping (directions available only)  if (state == 4) bonk_state_4();
  if (state == 4) bonk_state_4(); //  4 - Jumping down (directions available only)
  if (state == 5) bonk_state_5(); //  5 - Landing (all buttons disabled)
  if (state == 6) bonk_state_6(); //  6 - Bonking (all buttons disabled)
}

void bonk_state_0()
{
  /* this is Bonk's idle stance... if the tics are equal to 0, set his frame to default and his tic counter to 1 */
  if (tics == 0) spr_pattern(0x5100);
  tics = 1;
  /* most other states can be set in this state */
  if (held & JOY_LEFT)
  {
    /* start Bonk moving to the left! */
	spr_ctrl(FLIP_MAS,FLIP_X);
	tics = 0;
	frame = 0;
	state = 1;
	spr_pattern(0x5000);
	direction = DIR_LEFT;
  }
  if (held & JOY_RGHT)
  {
    /* start Bonk moving to the right! */
	spr_ctrl(FLIP_MAS,NO_FLIP);
	tics = 0;
	frame = 0;
	state = 1;
	spr_pattern(0x5000);
	direction = DIR_RIGHT;
  }
  if(pressed & JOY_B)
  {
    /* set Bonk in bonking state! */
	tics = 0;
	frame = 0;
	state = 6;
	spr_pattern(0x5700);
  }
  if(pressed & JOY_A)
  {
	/* make Bonk jump! */
	tics = 0;
	frame = 0;
	state = 2;
	spr_pattern(0x5400);
  }
}

void bonk_state_1()
{
  /* Bonk's walking state! */
  char stillwalking;

  stillwalking = 0;
  tics++;
  if (tics > 4)
  {
	tics = 0;
	frame++;
	if (frame > 5) frame = 0;
	if (frame == 0) spr_pattern(0x5100);
	if (frame == 1) spr_pattern(0x5200);
	if (frame == 2) spr_pattern(0x5300);
	if (frame == 3) spr_pattern(0x5200);
	if (frame == 4) spr_pattern(0x5100);
	if (frame == 5) spr_pattern(0x5000);
  }
  if (held & JOY_LEFT)
  {
    stillwalking = 1;
  }
  if(held & JOY_RGHT)
  {
    stillwalking = 1;
  }
  if(pressed & JOY_B)
  {
	/* set Bonk in bonking state! */
 	tics = 0;
	frame = 0;
	state = 6;
	spr_pattern(0x5700);
  }
  if(pressed & JOY_A)
  {
 	/* make Bonk jump! */
	tics = 0;
	frame = 0;
	state = 2;
	spr_pattern(0x5400);
  }

  if (stillwalking == 0)
  {
	state = 0;
	tics = 0;
	frame = 0;
  } else {
    move_in_world();
  }
}

void bonk_state_2()
{
  /* Bonk's pre-jump prep state! all controls are disabled in this one */
  tics++;
  if (tics > 4)
  {
	jumpspeed = 4;
	jumpdelta = 93;
	tics = 0; /* tics is going to be used in a different way in state 3... you'll see! */
	spr_pattern(0x5500);
	state = 3;	
  }
}

void bonk_state_3()
{
  /* Bonk's jumping up state! */
  bonky-=jumpspeed;
  /* here, we're going to use tics to determine how high Bonk has jumped, and compare it to the jump delta */
  tics+=jumpspeed;
  if (tics > jumpdelta)
  {
    /* if the total jump distance exceeds the jump delta, reduce the upward motion until it reaches zero */
	/* the reduction in upward motion makes the jump appear a little more natural than just immediately changing directions */
	spr_pattern(0x5600);
	jumpspeed--;
	if (jumpspeed<0)
	{
	  /* when the jump speed decreases past 0, we will need to set Bonk in state 4 */
	  tics = 0;
	  state = 4;
	  jumpspeed = 1;
	}
  }
  if (held & JOY_LEFT)
  {
	spr_ctrl(FLIP_MAS,FLIP_X);
	direction = DIR_LEFT;
	move_in_world();
  }
  if (held & JOY_RGHT)
  {
	spr_ctrl(FLIP_MAS,NO_FLIP);
	direction = DIR_RIGHT;
	move_in_world();
  }
  spr_y(bonky);
}

void bonk_state_4()
{
  /* Bonk's falling down state! */
  bonky+=jumpspeed;
  if (jumpspeed < 4) jumpspeed++;
  if (bonky > 175)
  {
	bonky = 176;
	frame = 0;
	tics = 0;
	state = 5;
	spr_pattern(0x5400);
  }
  if (held & JOY_LEFT)
  {
	spr_ctrl(FLIP_MAS,FLIP_X);
	direction = DIR_LEFT;
	move_in_world();
  }
  if (held & JOY_RGHT)
  {
	spr_ctrl(FLIP_MAS,NO_FLIP);
	direction = DIR_RIGHT;
	move_in_world();
  }
  spr_y(bonky);
}

void bonk_state_5()
{
  /* Bonk's landing state; same frame as state 2, but reverts to state 0 when finished rather than state 3 */
  tics++;
  if (tics > 4)
  {
	tics = 0;
	state = 0;
	frame = 0;
  }
}

void bonk_state_6()
{
  /* Bonk's bonking state! all controls are disabled in this one; this is just an animation handler/state controller */
  tics++;
  if (tics == 5)
  {
	spr_pattern(0x5800);
	/* we need to adjust Bonk's position, as this sprite is a few pixels to the left normally... this is why we need 'direction'! */
	if (direction == DIR_LEFT)
	{
	  spr_x(bonkx-8);
	} else {
	  spr_x(bonkx+8);
	}
  }
  if (tics > 10)
  {
	/* as this animation ends, restore the sprite to its normal posision and set the state back to 0 */
	spr_x(bonkx);
	tics = 0;
	state = 0;
  }
}

void move_in_world()
{
	if (direction == DIR_LEFT)
	{
		if (bonkx > -8) 
			bonkx--;
	}
	/* are we moving right? */
	if (direction == DIR_RIGHT)
	{
		if (bonkx < 232) 
			bonkx++;
	}
	spr_x(bonkx);
}
